@@ -119,13 +119,6 @@ def install_config_files(projdir, tmpdir):
     copied.extend(_copy_files((os.path.join(projdir, 'udisks/mount_options.conf'),),
                               '/etc/udisks2/', tmpdir))
 
-    # ZRAM module supplemental files
-    if os.path.exists(os.path.join(projdir, 'modules/zram/data/udisks2-zram-setup@.service')):
-        copied.extend(_copy_files((os.path.join(projdir, 'modules/zram/data/90-udisks2-zram.rules'),),
-                                  tgtdir, tmpdir))
-        copied.extend(_copy_files((os.path.join(projdir, 'modules/zram/data/udisks2-zram-setup@.service'),),
-                                  '/usr/lib/systemd/system/', tmpdir))
-
     return copied
 
 def restore_files(restore_list, tmpdir):
@@ -162,6 +155,7 @@ def _get_test_tags(test):
     """ Get test tags for single test case """
 
     tags = set()
+    tags.add(udiskstestcase.TestTags.ALL)
 
     # test failed to load, usually some ImportError or something really broken
     # in the test file, just return empty list and let it fail
@@ -185,8 +179,6 @@ def _get_test_tags(test):
     if getattr(test_fn, "extradeps", False) or getattr(test_fn.__self__, "extradeps", False):
         tags.add(udiskstestcase.TestTags.EXTRADEPS)
 
-    tags.add(udiskstestcase.TestTags.ALL)
-
     return tags
 
 
