@@ -41,7 +41,6 @@
 #include "udiskslinuxdevice.h"
 #include "udiskslinuxblock.h"
 #include "udisksfstabentry.h"
-#include "udisksfstabmonitor.h"
 #include "udiskscrypttabentry.h"
 #include "udiskscrypttabmonitor.h"
 #include "udisksspawnedjob.h"
@@ -163,26 +162,28 @@ update_metadata_size (UDisksLinuxEncrypted   *encrypted,
                       UDisksLinuxBlockObject *object)
 {
   UDisksLinuxDevice *device;
-  guint64 metadata_size;
+  BDCryptoLUKSInfo *info = NULL;
   GError *error = NULL;
 
   device = udisks_linux_block_object_get_device (object);
 
-  metadata_size = bd_crypto_luks_get_metadata_size (g_udev_device_get_device_file (device->udev_device),
-                                                    &error);
-
-  if (error != NULL)
-  {
-    udisks_warning ("Error getting '%s' metadata_size: %s (%s, %d)",
-                    g_udev_device_get_device_file (device->udev_device),
-                    error->message,
-                    g_quark_to_string (error->domain),
-                    error->code);
-    g_clear_error (&error);
-  }
+  info = bd_crypto_luks_info (g_udev_device_get_device_file (device->udev_device),
+                              &error);
+  if (!info)
+    {
+      udisks_warning ("Error getting '%s' metadata_size: %s (%s, %d)",
+                      g_udev_device_get_device_file (device->udev_device),
+                      error->message,
+                      g_quark_to_string (error->domain),
+                      error->code);
+      g_clear_error (&error);
+      udisks_encrypted_set_metadata_size (UDISKS_ENCRYPTED (encrypted), 0);
+    }
+  else
+    udisks_encrypted_set_metadata_size (UDISKS_ENCRYPTED (encrypted), info->metadata_size);
 
   g_object_unref (device);
-  udisks_encrypted_set_metadata_size (UDISKS_ENCRYPTED (encrypted), metadata_size);
+  bd_crypto_luks_info_free (info);
 }
 
 static void
@@ -193,17 +194,20 @@ update_cleartext_device (UDisksLinuxEncrypted   *encrypted,
   UDisksDaemon *daemon = udisks_linux_block_object_get_daemon (object);
   const gchar *encrypted_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (object));
 
-  /* wait_for_cleartext is used primarly in unlock but does exactly what we
+  /* wait_for_cleartext is used primarily in unlock but does exactly what we
      want -- returns a cleartext object for an encrypted object */
   cleartext_object = wait_for_cleartext_object (daemon, (gpointer) encrypted_path);
 
-  if (cleartext_object) {
+  if (cleartext_object)
+    {
       udisks_encrypted_set_cleartext_device (UDISKS_ENCRYPTED (encrypted),
                                              g_dbus_object_get_object_path (G_DBUS_OBJECT (cleartext_object)));
-  }
-  else {
-    udisks_encrypted_set_cleartext_device (UDISKS_ENCRYPTED (encrypted), "/");
-  }
+      g_object_unref (cleartext_object);
+    }
+  else
+    {
+      udisks_encrypted_set_cleartext_device (UDISKS_ENCRYPTED (encrypted), "/");
+    }
 }
 
 /**
@@ -226,14 +230,17 @@ udisks_linux_encrypted_update (UDisksLinuxEncrypted   *encrypted,
 
   /* set block type according to hint_encryption_type */
   if (udisks_linux_block_is_unknown_crypto (block))
-  {
-    if (g_strcmp0(udisks_encrypted_get_hint_encryption_type(UDISKS_ENCRYPTED(encrypted)), "TCRYPT") == 0)
-      udisks_block_set_id_type (block, "crypto_TCRYPT");
-  }
+    {
+      if (g_strcmp0 (udisks_encrypted_get_hint_encryption_type (UDISKS_ENCRYPTED (encrypted)), "TCRYPT") == 0)
+        udisks_block_set_id_type (block, "crypto_TCRYPT");
+    }
 
-  update_metadata_size (encrypted, object);
+  if (udisks_linux_block_is_luks (block))
+    update_metadata_size (encrypted, object);
 
   udisks_linux_block_encrypted_unlock (block);
+
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (encrypted));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -311,9 +318,9 @@ has_option (const gchar *options,
           goto out;
         }
     }
-  g_strfreev (tokens);
 
  out:
+  g_strfreev (tokens);
   return ret;
 }
 
@@ -329,7 +336,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
   UDisksObject *object = NULL;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksState *state;
+  UDisksState *state = NULL;
   gchar *name = NULL;
   UDisksObject *cleartext_object = NULL;
   UDisksBlock *cleartext_block;
@@ -344,7 +351,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
   gsize crypttab_passphrase_len = 0;
   gchar *crypttab_options = NULL;
   gchar *device = NULL;
-  gchar *old_hint_encryption_type;
+  gchar *old_hint_encryption_type = NULL;
   gboolean read_only = FALSE;
   gboolean is_hidden = FALSE;
   gboolean is_system = FALSE;
@@ -354,8 +361,10 @@ handle_unlock (UDisksEncrypted        *encrypted,
   const gchar *keyfiles[MAX_TCRYPT_KEYFILES] = {};
   CryptoJobData data;
   gboolean is_luks;
+  gboolean is_bitlk;
   gboolean handle_as_tcrypt;
   void *open_func;
+  const gchar *uuid = NULL;
 
   object = udisks_daemon_util_dup_object (encrypted, &error);
   if (object == NULL)
@@ -368,8 +377,12 @@ handle_unlock (UDisksEncrypted        *encrypted,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   state = udisks_daemon_get_state (daemon);
   is_luks = udisks_linux_block_is_luks (block);
+  is_bitlk = udisks_linux_block_is_bitlk (block);
   handle_as_tcrypt = udisks_linux_block_is_tcrypt (block) || udisks_linux_block_is_unknown_crypto (block);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   /* get TCRYPT options */
   if (handle_as_tcrypt)
     {
@@ -400,12 +413,12 @@ handle_unlock (UDisksEncrypted        *encrypted,
    */
 
   /* Fail if the device is not a LUKS or possible TCRYPT device */
-  if (!(is_luks || handle_as_tcrypt))
+  if (!(is_luks || is_bitlk || handle_as_tcrypt))
     {
       g_dbus_method_invocation_return_error (invocation,
                                              UDISKS_ERROR,
                                              UDISKS_ERROR_FAILED,
-                                             "Device %s does not appear to be a LUKS or TCRYPT device",
+                                             "Device %s does not appear to be a LUKS, BITLK or TCRYPT device",
                                              udisks_block_get_device (block));
       goto out;
     }
@@ -431,7 +444,6 @@ handle_unlock (UDisksEncrypted        *encrypted,
     }
 
   /* we need the uid of the caller for the unlocked-crypto-dev file */
-  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -440,7 +452,6 @@ handle_unlock (UDisksEncrypted        *encrypted,
     }
 
   /* check if in crypttab file */
-  error = NULL;
   if (!check_crypttab (block,
                        TRUE,
                        &is_in_crypttab,
@@ -515,6 +526,14 @@ handle_unlock (UDisksEncrypted        *encrypted,
   else {
     if (is_luks)
       name = g_strdup_printf ("luks-%s", udisks_block_get_id_uuid (block));
+    else if (is_bitlk)
+      {
+        uuid = udisks_block_get_id_uuid (block);
+        if (uuid && g_strcmp0 (uuid, "") != 0)
+          name = g_strdup_printf ("bitlk-%s", uuid);
+        else
+          name = g_strdup_printf ("bitlk-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
+      }
     else
       /* TCRYPT devices don't have a UUID, so we use the device number instead */
       name = g_strdup_printf ("tcrypt-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
@@ -528,12 +547,15 @@ handle_unlock (UDisksEncrypted        *encrypted,
    * update triggered by the unlock. */
   if (is_luks)
     udisks_encrypted_set_hint_encryption_type (encrypted, "LUKS");
+  else if (is_bitlk)
+    udisks_encrypted_set_hint_encryption_type (encrypted, "BITLK");
   else
     udisks_encrypted_set_hint_encryption_type (encrypted, "TCRYPT");
 
   device = udisks_block_dup_device (block);
 
-  /* TODO: support reading a 'readonly' option from @options */
+  /* unlock as read-only if specified in @options or if the device itself is read-only */
+  g_variant_lookup (options, "read-only", "b", &read_only);
   if (udisks_block_get_read_only (block))
     read_only = TRUE;
 
@@ -548,6 +570,8 @@ handle_unlock (UDisksEncrypted        *encrypted,
 
   if (is_luks)
     open_func = luks_open_job_func;
+  else if (is_bitlk)
+    open_func = bitlk_open_job_func;
   else
     open_func = tcrypt_open_job_func;
 
@@ -568,6 +592,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
                                              "Error unlocking %s: %s",
                                              udisks_block_get_device (block),
                                              error->message);
+      g_clear_error (&error);
 
       /* Restore the old encryption type if the unlock failed, because
        * in this case we don't know for sure if we used the correct
@@ -576,27 +601,20 @@ handle_unlock (UDisksEncrypted        *encrypted,
       udisks_linux_block_encrypted_unlock (block);
       goto out;
     }
-  else
-    {
-      /* We have to free old_hint_encryption_type if and only if it was not
-       * used in udisks_encrypted_set_hint_encryption_type() */
-      g_free (old_hint_encryption_type);
-    }
 
   udisks_linux_block_encrypted_unlock (block);
 
   /* Determine the resulting cleartext object */
-  error = NULL;
   cleartext_object = udisks_daemon_wait_for_object_sync (daemon,
                                                          wait_for_cleartext_object,
                                                          g_strdup (g_dbus_object_get_object_path (G_DBUS_OBJECT (object))),
                                                          g_free,
-                                                         20, /* timeout_seconds */
+                                                         UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                          &error);
   if (cleartext_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for cleartext object after unlocking %s",
+                      "Error waiting for cleartext object after unlocking '%s': ",
                       udisks_block_get_device (block));
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -616,16 +634,26 @@ handle_unlock (UDisksEncrypted        *encrypted,
                                         g_udev_device_get_sysfs_attr (cleartext_device->udev_device, "dm/uuid"),
                                         caller_uid);
 
+  /* ensure property changes are sent before the method return */
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (encrypted));
+
   udisks_encrypted_complete_unlock (encrypted,
                                     invocation,
                                     g_dbus_object_get_object_path (G_DBUS_OBJECT (cleartext_object)));
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device);
   g_free (crypttab_name);
   g_free (crypttab_passphrase);
   g_free (crypttab_options);
   g_free (name);
+  g_free (old_hint_encryption_type);
+  if (keyfiles_variant)
+    g_variant_unref (keyfiles_variant);
   g_clear_object (&cleartext_device);
   g_clear_object (&cleartext_object);
   g_clear_object (&object);
@@ -658,6 +686,7 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   gchar *cleartext_path = NULL;
   void *close_func;
   gboolean is_luks;
+  gboolean is_bitlk;
   gboolean handle_as_tcrypt;
 
   object = udisks_daemon_util_dup_object (encrypted, error);
@@ -671,6 +700,7 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   state = udisks_daemon_get_state (daemon);
   is_luks = udisks_linux_block_is_luks (block);
+  is_bitlk = udisks_linux_block_is_bitlk (block);
   handle_as_tcrypt = udisks_linux_block_is_tcrypt (block) || udisks_linux_block_is_unknown_crypto (block);
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
@@ -679,12 +709,12 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
    */
 
   /* Fail if the device is not a LUKS or possible TCRYPT device */
-  if (!(is_luks || handle_as_tcrypt))
+  if (!(is_luks || is_bitlk || handle_as_tcrypt))
     {
       g_set_error (error,
                    UDISKS_ERROR,
                    UDISKS_ERROR_FAILED,
-                   "Device %s does not appear to be a LUKS or TCRYPT device",
+                   "Device %s does not appear to be a LUKS, BITLK or TCRYPT device",
                    udisks_block_get_device (block));
       ret = FALSE;
       goto out;
@@ -755,10 +785,12 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
     }
 
   device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
-  data.map_name = g_udev_device_get_sysfs_attr (device->udev_device, "dm/name");
+  data.map_name = (gchar *) g_udev_device_get_sysfs_attr (device->udev_device, "dm/name");
 
   if (is_luks)
     close_func = luks_close_job_func;
+  else if (is_bitlk)
+    close_func = bitlk_close_job_func;
   else
     close_func = tcrypt_close_job_func;
 
@@ -793,7 +825,7 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
                                                          wait_for_cleartext_object,
                                                          cleartext_path,
                                                          NULL,
-                                                         10,
+                                                         UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                          &loc_error))
     {
       g_set_error (error,
@@ -829,12 +861,30 @@ handle_lock (UDisksEncrypted        *encrypted,
              GVariant               *options)
 {
   GError *error = NULL;
+  UDisksObject *object = NULL;
+  UDisksDaemon *daemon = NULL;
+  UDisksState *state = NULL;
+
+  object = udisks_daemon_util_dup_object (encrypted, NULL);
+  if (object != NULL)
+    {
+      daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+      state = udisks_daemon_get_state (daemon);
+      udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+      udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+    }
 
   if (!udisks_linux_encrypted_lock (UDISKS_LINUX_ENCRYPTED (encrypted), invocation, options, &error))
     g_dbus_method_invocation_take_error (invocation, error);
   else
     udisks_encrypted_complete_lock (encrypted, invocation);
 
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
+  g_clear_object (&object);
+
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
@@ -851,6 +901,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
   UDisksObject *object = NULL;
   UDisksBlock *block;
   UDisksDaemon *daemon;
+  UDisksState *state = NULL;
   uid_t caller_uid;
   const gchar *action_id;
   GError *error = NULL;
@@ -866,6 +917,10 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
+
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
    *
@@ -883,7 +938,6 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
       goto out;
     }
 
-  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -897,8 +951,8 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
   if (udisks_block_get_hint_system (block) &&
       !(udisks_daemon_util_setup_by_user (daemon, object, caller_uid)))
     action_id = "org.freedesktop.udisks2.encrypted-change-passphrase-system";
-  //if (is_in_crypttab)
-  //  action_id = "org.freedesktop.udisks2.encrypted-unlock-crypttab";
+  /*  if (is_in_crypttab)
+        action_id = "org.freedesktop.udisks2.encrypted-unlock-crypttab";  */
   if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                     object,
                                                     action_id,
@@ -951,6 +1005,10 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
   udisks_encrypted_complete_change_passphrase (encrypted, invocation);
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device);
   udisks_string_wipe_and_free (data.passphrase);
   udisks_string_wipe_and_free (data.new_passphrase);
@@ -973,22 +1031,28 @@ handle_resize (UDisksEncrypted       *encrypted,
   UDisksObject *cleartext_object = NULL;
   UDisksBlock *cleartext_block;
   UDisksDaemon *daemon;
+  UDisksState *state = NULL;
   uid_t caller_uid;
   const gchar *action_id = NULL;
   const gchar *message = NULL;
   GError *error = NULL;
   UDisksBaseJob *job = NULL;
   GString *effective_passphrase = NULL;
+  BDCryptoKeyslotContext *context = NULL;
 
   object = udisks_daemon_util_dup_object (encrypted, &error);
   if (object == NULL)
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
       goto out;
     }
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
+
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
 
   /* Fail if the device is not a LUKS device */
   if (!(g_strcmp0 (udisks_block_get_id_usage (block), "crypto") == 0 &&
@@ -1002,11 +1066,9 @@ handle_resize (UDisksEncrypted       *encrypted,
       goto out;
     }
 
-  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -1057,6 +1119,30 @@ handle_resize (UDisksEncrypted       *encrypted,
                                                      invocation))
     goto out;
 
+  if (udisks_variant_lookup_binary (options, "keyfile_contents", &effective_passphrase))
+    ;
+  else if (udisks_variant_lookup_binary (options, "passphrase", &effective_passphrase))
+    ;
+  else
+    effective_passphrase = NULL;
+
+  if (effective_passphrase)
+    {
+      context = bd_crypto_keyslot_context_new_passphrase ((const guint8 *) effective_passphrase->str,
+                                                          effective_passphrase->len,
+                                                          &error);
+      if (!context)
+        {
+          g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             "Error resizing encrypted device %s: %s",
+                                             udisks_block_get_device (cleartext_block),
+                                             error->message);
+          goto out;
+        }
+    }
+
   job = udisks_daemon_launch_simple_job (daemon,
                                          UDISKS_OBJECT (object),
                                          "encrypted-resize",
@@ -1069,19 +1155,13 @@ handle_resize (UDisksEncrypted       *encrypted,
       goto out;
     }
 
-  if (udisks_variant_lookup_binary (options, "keyfile_contents", &effective_passphrase))
-    ;
-  else if (udisks_variant_lookup_binary (options, "passphrase", &effective_passphrase))
-    ;
-  else
-    effective_passphrase = NULL;
+  udisks_linux_block_encrypted_lock (block);
 
   /* TODO: implement progress parsing for udisks_job_set_progress(_valid) */
-  if (! bd_crypto_luks_resize_luks2_blob (udisks_block_get_device (cleartext_block),
-                                          size / 512,
-                                          effective_passphrase ? (const guint8*) effective_passphrase->str : NULL,
-                                          effective_passphrase ? effective_passphrase->len : 0,
-                                          &error))
+  if (! bd_crypto_luks_resize (udisks_block_get_device (cleartext_block),
+                               size / 512,
+                               context,
+                               &error))
     {
       g_dbus_method_invocation_return_error (invocation,
                                              UDISKS_ERROR,
@@ -1090,17 +1170,25 @@ handle_resize (UDisksEncrypted       *encrypted,
                                              udisks_block_get_device (cleartext_block),
                                              error->message);
       udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      udisks_linux_block_encrypted_unlock (block);
       goto out;
     }
 
+  udisks_linux_block_encrypted_unlock (block);
+
   udisks_encrypted_complete_resize (encrypted, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_clear_object (&cleartext_object);
   g_clear_object (&object);
   g_clear_error (&error);
   udisks_string_wipe_and_free (effective_passphrase);
+  bd_crypto_keyslot_context_free (context);
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
